# -*- encoding: utf-8 -*-
#
#  Created by Joel Grand-Guillaume
#
#  Copyright (c) 2009 CamptoCamp. All rights reserved.
##############################################################################
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
import time
from mx import DateTime
import netsvc
from osv import fields
from osv import osv
from datetime import datetime


# Class wich will create all languages, function and views for Excel Reporting
# 3 level:
#            - Function and languages definition
#            - Araud Wuest works about multi level recusrion in many2many context
#            - Views
# All kind of things will be instanciated in the init function
class report_excel_c2c_base_function(osv.osv):
    _name = "report.excel.c2c.base.function"
    _description = "Base class and function for Excel reporting"
    _auto = False
    _log_access = False

    def init(self, cr):
        
        
        #--------------------------------------------------------------------------------------
        #   LANGUAGE PL PGSQL !
        #--------------------------------------------------------------------------------------
        try:
            cr.execute("""CREATE TRUSTED PROCEDURAL LANGUAGE plpgsql
            HANDLER plpgsql_call_handler
            VALIDATOR plpgsql_validator;""")
            cr.commit()
        # Pass if failed cause it's already installed...
        except:
            cr.rollback()
            pass
        
        #--------------------------------------------------------------------------------------
        #   Functions !
        #--------------------------------------------------------------------------------------
        # Generate series for working days between dates
        # Take :
        # first_date as date
        # last_date as date
        #
        # Retrun
        # setof date
        cr.execute("""CREATE OR REPLACE FUNCTION non_weekends_between(
            first_date DATE, /* $1 */
            last_date DATE   /* $2 */
        )
        RETURNS SETOF date
        LANGUAGE sql
        AS
        $$
        SELECT
            $1 + s.i
        FROM generate_series(
            0,
            $2 - $1
        ) AS s(i)
        WHERE
            extract(
                DOW
            FROM
                $1 + s.i
            ) NOT IN (
                0,  /* Sunday   */
                6   /* Saturday */
            );
        $$;
        """)
        cr.commit()
        
        # Return the start date of a month
        # Take :
        # date as date
        #
        # Retrun
        # first day of month as date
        cr.execute("""CREATE OR REPLACE FUNCTION month_start(
            date
        )
        RETURNS date AS
        $BODY$
            select date_trunc ('month', $1)::date;
            $BODY$
        LANGUAGE 'sql' VOLATILE
        COST 100;
        """)
        cr.commit()
        
        # Return the end date of a month
        # Take :
        # date as date
        #
        # Retrun
        # last day of month as date
        cr.execute("""CREATE OR REPLACE FUNCTION month_end(
            date
        )
        RETURNS date AS
        $BODY$
            select (month_start ($1) - 1 + interval '1 month')::date;
            $BODY$
        LANGUAGE 'sql' VOLATILE
        COST 100;
        """)
        cr.commit()
        
        
        # Return the number of working day in the month of the date passed in args
        # Take :
        # the_date as date
        #
        # Retrun
        # number of working days as int
        cr.execute("""CREATE OR REPLACE FUNCTION working_day_in_month(the_date date)
        RETURNS bigint AS
        $BODY$
                select count(*) 
                from(
                    select non_weekends_between(foo.month_start,foo.month_end) 
                    from
                        (
                        select * 
                        from month_start($1),month_end($1)
                        ) as foo
                ) as foo2
                $BODY$
        LANGUAGE 'sql' VOLATILE
        COST 100;
        """)
        cr.commit()
        


        # count working day between dates
        # Take :
        # first_date as date
        # last_date as date
        #
        # Retrun
        # number of working days as int
        cr.execute("""CREATE OR REPLACE FUNCTION working_day_in_dates (
            first_date DATE, /* $1 */
            last_date DATE   /* $2 */   )
        RETURNS int8
        LANGUAGE sql
        AS
        $$
        select count(*) 
        from(
            select non_weekends_between($1,$2)  as foo
        ) as foo2
        $$;
        """)
        cr.commit()
        
        # ------------------------------------------------------------------------------------
        # ACCOUNT ACCOUNT
        # ------------------------------------------------------------------------------------
        
        # Recursive function that return a full name for the analytic_account_id
        # (full name is name_parent_level1 + '->-' + parent_level_2 + ... + analytic_account_id_name)
        # Take :
        # analytic_account_id as int
        #
        # Retrun
        # Full name with all parent as varchar(2048)
        cr.execute("""CREATE OR REPLACE FUNCTION cp_getaccountfullname(bigint)
          RETURNS character varying AS
        $BODY$DECLARE
        itemid ALIAS FOR $1;
        itemfullname varchar(2048);
        itemrecord RECORD;
        BEGIN
            SELECT s.* INTO itemrecord FROM account_account s  where id=itemid;
             itemfullname =  itemrecord.name;
             IF itemrecord.parent_id IS NOT NULL  THEN
                   itemfullname := cp_getaccountfullname(itemrecord.parent_id) || '->-' || itemfullname ;
                   RETURN itemfullname;
             ELSE
                   RETURN itemfullname;
            END IF;
        END$BODY$
          LANGUAGE 'plpgsql' VOLATILE
          COST 100;""")
        cr.commit()
        
        
        # Function that call the cp_getitemfullname for the passed id, and return a setof name
        # Take :
        # analytic_account_id as int
        #
        # Retrun
        # Set of Full name with all parent as varchar(2048) 
        # (even if it's a set of only one, at least you have it like this:
        # ?Column?
        # --------
        # blablablabl ->- uiuiui -> name
        cr.execute("""CREATE OR REPLACE FUNCTION cp_getsetofaccountfromfullname(bigint)
          RETURNS text[] AS
        $BODY$
        select string_to_array(temp,'->-') from cp_getaccountfullname($1) as temp
        $BODY$
          LANGUAGE 'sql' VOLATILE
          COST 100;""")
        cr.commit()
        
        
        
        # Recursive function that return a full name for the analytic_account_id WITH THE PROJECT CODE !!!!
        
        # (full name is code_name_parent_level1 + '->-' + code_name_parent_level_2 + ... + code_analytic_account_id_name)
        # Take :
        # analytic_account_id as int
        #
        # Retrun
        # Full code+name with all parent as varchar(2048)
        cr.execute("""CREATE OR REPLACE FUNCTION cp_getaccountfullnamecode(bigint)
          RETURNS character varying AS
        $BODY$DECLARE
        itemid ALIAS FOR $1;
        itemfullname varchar(2048);
        itemrecord RECORD;
        BEGIN
            SELECT s.* INTO itemrecord FROM account_account s  where id=itemid;
             itemfullname =  itemrecord.code || ' - ' || itemrecord.name;
             IF itemrecord.parent_id IS NOT NULL  THEN
                   itemfullname := cp_getaccountfullname(itemrecord.parent_id) || '->-' || itemfullname ;
                   RETURN itemfullname;
             ELSE
                   RETURN itemfullname;
            END IF;
        END$BODY$
          LANGUAGE 'plpgsql' VOLATILE
          COST 100;""")
        cr.commit()
        
        
        # Function that call the cp_getitemfullname for the passed id, and return a setof name WITH THE PROJECT CODE !!!!
        # Take :
        # analytic_account_id as int
        #
        # Retrun
        # Set of Full code+name with all parent as varchar(2048) 
        # (even if it's a set of only one, at least you have it like this:
        # ?Column?
        # --------
        # blablablabl ->- uiuiui -> name
        cr.execute("""CREATE OR REPLACE FUNCTION cp_getsetofaccountfromfullnamecode(bigint)
          RETURNS text[] AS
        $BODY$
        select string_to_array(temp,'->-') from cp_getaccountfullnamecode($1) as temp
        $BODY$
          LANGUAGE 'sql' VOLATILE
          COST 100;""")
        cr.commit()
        
        # ------------------------------------------------------------------------------------
        # PRODUCT_CATEGORY
        # ------------------------------------------------------------------------------------
        
        # Recursive function that return a full name for the analytic_account_id
        # (full name is name_parent_level1 + '->-' + parent_level_2 + ... + analytic_account_id_name)
        # Take :
        # analytic_account_id as int
        #
        # Retrun
        # Full name with all parent as varchar(2048)
        cr.execute("""CREATE OR REPLACE FUNCTION cp_getproductcategfullname(bigint)
          RETURNS character varying AS
        $BODY$DECLARE
        itemid ALIAS FOR $1;
        itemfullname varchar(2048);
        itemrecord RECORD;
        BEGIN
            SELECT s.* INTO itemrecord FROM product_category s  where id=itemid;
             itemfullname =  itemrecord.name;
             IF itemrecord.parent_id IS NOT NULL  THEN
                   itemfullname := cp_getproductcategfullname(itemrecord.parent_id) || '->-' || itemfullname ;
                   RETURN itemfullname;
             ELSE
                   RETURN itemfullname;
            END IF;
        END$BODY$
          LANGUAGE 'plpgsql' VOLATILE
          COST 100;""")
        cr.commit()
        
        # Function that call the cp_getitemfullname for the passed id, and return a setof name
        # Take :
        # analytic_account_id as int
        #
        # Retrun
        # Set of Full name with all parent as varchar(2048) 
        # (even if it's a set of only one, at least you have it like this:
        # ?Column?
        # --------
        # blablablabl ->- uiuiui -> name
        cr.execute("""CREATE OR REPLACE FUNCTION cp_getsetproductcategfullname(bigint)
          RETURNS text[] AS
        $BODY$
        select string_to_array(temp,'->-') from cp_getproductcategfullname($1) as temp
        $BODY$
          LANGUAGE 'sql' VOLATILE
          COST 100;""")
        cr.commit()
        
                # 
                # 
                # # Recursive function that return a full name for the analytic_account_id WITH THE PROJECT CODE !!!!
                # 
                # # (full name is code_name_parent_level1 + '->-' + code_name_parent_level_2 + ... + code_analytic_account_id_name)
                # # Take :
                # # analytic_account_id as int
                # #
                # # Retrun
                # # Full code+name with all parent as varchar(2048)
                # cr.execute("""CREATE OR REPLACE FUNCTION cp_getaccountfullnamecode(bigint)
                #   RETURNS character varying AS
                # $BODY$DECLARE
                # itemid ALIAS FOR $1;
                # itemfullname varchar(2048);
                # itemrecord RECORD;
                # BEGIN
                #     SELECT s.* INTO itemrecord FROM account_account s  where id=itemid;
                #      itemfullname =  itemrecord.code || ' - ' || itemrecord.name;
                #      IF itemrecord.parent_id IS NOT NULL  THEN
                #            itemfullname := cp_getaccountfullname(itemrecord.parent_id) || '->-' || itemfullname ;
                #            RETURN itemfullname;
                #      ELSE
                #            RETURN itemfullname;
                #     END IF;
                # END$BODY$
                #   LANGUAGE 'plpgsql' VOLATILE
                #   COST 100;""")
                # cr.commit()
                # 
                # 
                # # Function that call the cp_getitemfullname for the passed id, and return a setof name WITH THE PROJECT CODE !!!!
                # # Take :
                # # analytic_account_id as int
                # #
                # # Retrun
                # # Set of Full code+name with all parent as varchar(2048) 
                # # (even if it's a set of only one, at least you have it like this:
                # # ?Column?
                # # --------
                # # blablablabl ->- uiuiui -> name
                # cr.execute("""CREATE OR REPLACE FUNCTION cp_getsetofaccountfromfullnamecode(bigint)
                #   RETURNS text[] AS
                # $BODY$
                # select string_to_array(temp,'->-') from cp_getaccountfullnamecode($1) as temp
                # $BODY$
                #   LANGUAGE 'sql' VOLATILE
                #   COST 100;""")
                # cr.commit()
                # 
                #         
        
        # ------------------------------------------------------------------------------------
        # ANALYTIC ACCOUNT FUNCTION
        # ------------------------------------------------------------------------------------
        #
        # We don't change the name of function for retro-compatibility purpose
        
        
        
        # Recursive function that return a full name for the analytic_account_id
        # (full name is name_parent_level1 + '->-' + parent_level_2 + ... + analytic_account_id_name)
        # Take :
        # analytic_account_id as int
        #
        # Retrun
        # Full name with all parent as varchar(2048)
        cr.execute("""CREATE OR REPLACE FUNCTION cp_getitemfullname(bigint)
          RETURNS character varying AS
        $BODY$DECLARE
        itemid ALIAS FOR $1;
        itemfullname varchar(2048);
        itemrecord RECORD;
        BEGIN
            SELECT s.* INTO itemrecord FROM account_analytic_account s  where id=itemid;
             itemfullname =  itemrecord.name;
             IF itemrecord.parent_id IS NOT NULL  THEN
                   itemfullname := cp_getitemfullname(itemrecord.parent_id) || '->-' || itemfullname ;
                   RETURN itemfullname;
             ELSE
                   RETURN itemfullname;
            END IF;
        END$BODY$
          LANGUAGE 'plpgsql' VOLATILE
          COST 100;""")
        cr.commit()
        

        # Function that call the cp_getitemfullname for the passed id, and return a setof name
        # Take :
        # analytic_account_id as int
        #
        # Retrun
        # Set of Full name with all parent as varchar(2048) 
        # (even if it's a set of only one, at least you have it like this:
        # ?Column?
        # --------
        # blablablabl ->- uiuiui -> name
        cr.execute("""CREATE OR REPLACE FUNCTION cp_getsetofitemfromfullname(bigint)
          RETURNS text[] AS
        $BODY$
        select string_to_array(temp,'->-') from cp_getitemfullname($1) as temp
        $BODY$
          LANGUAGE 'sql' VOLATILE
          COST 100;""")
        cr.commit()



        # Recursive function that return a full name for the analytic_account_id WITH THE PROJECT CODE !!!!

        # (full name is code_name_parent_level1 + '->-' + code_name_parent_level_2 + ... + code_analytic_account_id_name)
        # Take :
        # analytic_account_id as int
        #
        # Retrun
        # Full code+name with all parent as varchar(2048)
        cr.execute("""CREATE OR REPLACE FUNCTION cp_getitemfullnamecode(bigint)
          RETURNS character varying AS
        $BODY$DECLARE
        itemid ALIAS FOR $1;
        itemfullname varchar(2048);
        itemrecord RECORD;
        BEGIN
            SELECT s.* INTO itemrecord FROM account_analytic_account s  where id=itemid;
             itemfullname =  itemrecord.code || ' - ' || itemrecord.name;
             IF itemrecord.parent_id IS NOT NULL  THEN
                   itemfullname := cp_getitemfullname(itemrecord.parent_id) || '->-' || itemfullname ;
                   RETURN itemfullname;
             ELSE
                   RETURN itemfullname;
            END IF;
        END$BODY$
          LANGUAGE 'plpgsql' VOLATILE
          COST 100;""")
        cr.commit()
        
        
        # Function that call the cp_getitemfullname for the passed id, and return a setof name WITH THE PROJECT CODE !!!!
        # Take :
        # analytic_account_id as int
        #
        # Retrun
        # Set of Full code+name with all parent as varchar(2048) 
        # (even if it's a set of only one, at least you have it like this:
        # ?Column?
        # --------
        # blablablabl ->- uiuiui -> name
        cr.execute("""CREATE OR REPLACE FUNCTION cp_getsetofitemfromfullnamecode(bigint)
          RETURNS text[] AS
        $BODY$
        select string_to_array(temp,'->-') from cp_getitemfullnamecode($1) as temp
        $BODY$
          LANGUAGE 'sql' VOLATILE
          COST 100;""")
        cr.commit()
        
        
        #--------------------------------------------------------------------------------------
        # Multi-Recursion By Arnaud Wuest
        #--------------------------------------------------------------------------------------
        
        # Here I put all the SQL code by Arnaud Wuest in one single cr.execute function
        
        cr.execute("""
        
        --
        --
        -- USAGE (read it!)
        --
        --

        -- Be sure plpgsql language is create, otherwise uncomment this line:
        -- CREATE LANGUAGE plpgsql;

        -- Use the methods under the "API" heading at the bottom of this file. 
        -- methods require a param "source" that must be a view name (as text) or a full query (also as text). 

        -- When run, this source must return all elements'ids associated their corresponding values and corresponding parent.
        -- if the element has several parents, the source must return for each id, as many lines as it has parents.
        -- Each of these lines must contain the id's coresponding value
        -- If the element has no parents, parent_id must contain NULL.
        -- The "val" value must never be NULL 
        -- *** In short: you'll have to use LEFT JOIN and filter on NULL values ***

        -- The returned lines' field names must match the type "tree_data" (see "TYPES" heading juste below this text)



        --
        --
        -- TYPES
        --
        --

        -- Source data type. The source must return data of this struct.
        DROP TYPE IF EXISTS tree_data CASCADE;
        CREATE TYPE tree_data AS (
        id integer,
        parent_id integer,
        val varchar -- what will appear in result's pathes
        );

        -- Result data type. The main method return this kind of data
        DROP TYPE IF EXISTS tree_path CASCADE;
        CREATE TYPE tree_path AS (
        id integer,
        path varchar[] -- path from the greatest parent to the id (included)
        );



        --
        --
        -- INTERNAL FUNCTIONS
        --
        --



        -- Extract the data from the source information.
        -- pass to this method a query (as text) or a view's name (also as text)
        CREATE OR REPLACE FUNCTION tree_get_formated_data(source text)
        RETURNS  SETOF tree_data AS
        $BODY$
        DECLARE rec tree_data;
        DECLARE query text;
        BEGIN
            -- source param contain a only a view name
            IF position('select' in lower(source)) > 0 THEN
                query = source;
            -- source param contain a full query
            ELSE 
                query = 'SELECT id, parent_id, val FROM ' || quote_ident(source);
            END IF;

            -- return results
            FOR rec IN EXECUTE query LOOP
                RETURN NEXT rec;
            END LOOP;
        END;
        $BODY$
        LANGUAGE plpgsql VOLATILE;


        -- Return the parents' ids of an item
        CREATE OR REPLACE FUNCTION tree_get_parents(item_id integer, source text)
        RETURNS SETOF integer AS 
        $BODY$
            --get the parent_ids corresponding to the id from the source's datas
            SELECT distinct(parent_id) AS id
            FROM tree_get_formated_data($2) 
            WHERE id = $1 AND parent_id IS NOT NULL
        $BODY$
        LANGUAGE 'sql' VOLATILE;


        -- Return the item's value
        CREATE OR REPLACE FUNCTION tree_get_value(item_id integer, source text)
        RETURNS varchar as 
        $BODY$
            --get the value corresponding to the id from the source's datas
            SELECT val from
            tree_get_formated_data($2)
            WHERE id = $1
        $BODY$
        LANGUAGE 'sql' VOLATILE;




        -- Complete a list of pathes with the given value
        -- for exemple, give it pathes = {gnagna; gnogno/bloblo} and val = "trululu"
        -- it will return  {gnagna/trululu; gnogno/bloblo/trululu}
        CREATE OR REPLACE FUNCTION tree_complete_pathes(pathes varchar[], val varchar)
        RETURNS varchar[] AS
        $BODY$
        DECLARE i integer;
        DECLARE newPathes varchar[];
        BEGIN

            -- for each received path, concat the current value 
            i := 1;
            LOOP 
                -- stop if the pathes is empty, we went through all pathes
                exit when array_upper(pathes, 1) is null or i > array_upper(pathes, 1);        

                newPathes := array_append(newPathes, cast( val || '^/~' || pathes[i] as varchar));

                i := i + 1;
            END LOOP;
            return newPathes;

        END;
        $BODY$
        LANGUAGE plpgsql VOLATILE;



        -- For a given id, return  all pathes to these parents
        CREATE OR REPLACE FUNCTION tree_get_subpathes(child_id INTEGER, pathes varchar[], source text)
        RETURNS varchar[] AS 
        $BODY$
        DECLARE parent record;
        DECLARE val varchar;
        DECLARE result varchar[];
        DECLARE hasParents boolean;
        DECLARE newPathes varchar[];

        BEGIN
            hasParents := false;

            val := tree_get_value(child_id, source);

            --for the first call, init the pathes array with the id's corresponding value
            if array_upper(pathes, 1) is null or array_upper(pathes, 1) = 0 then
                newPathes:= array_append(newPathes, val);

            --for subsequent calls, complete the existing pathes
            else
                -- for each received path, concat the current value
                newPathes := tree_complete_pathes(pathes, val);
            end if;

            -- call the same method for each parents
            FOR parent IN select tree_get_parents as id from tree_get_parents(child_id, source) LOOP
                hasParents := true;
                -- add parent results to this item's result
                result := array_cat(result, tree_get_subpathes(parent.id, newPathes, source));        
            END LOOP;

            --return parents' result
            if hasParents THEN
                return result;
            --no parents, return its own results
            else 
                return newPathes;
            end if;

        END;    
        $BODY$
        LANGUAGE plpgsql VOLATILE;






        --
        --
        -- API
        --
        --


        -- Return all possible parents pathes for a given node
        CREATE OR REPLACE FUNCTION tree_get_pathes(item_id integer, source text)
        RETURNS SETOF tree_path AS 
        $BODY$
        DECLARE rec tree_path;
        DECLARE pathes varchar[];
        DECLARE subpathes varchar[];
        declare i integer;
        BEGIN
            -- get all pathes
            subpathes = tree_get_subpathes(item_id, pathes, source);

            -- transform array of text-pathes into rows of array-pathes
            i := 1;
            loop
                exit when array_upper(subpathes, 1) is null or i > array_upper(subpathes, 1);
                rec.id = item_id;
                rec.path = string_to_array(subpathes[i], '^/~');
                i := i + 1;
                RETURN NEXT rec;

            end LOOP;
        RETURN;
        END;
        $BODY$
        LANGUAGE plpgsql VOLATILE;



        -- Return all possible pathes for all ids from the source 
        CREATE OR REPLACE FUNCTION tree_get_all_pathes(source text)
        RETURNS SETOF tree_path AS 
        $BODY$
        DECLARE browse_rec tree_path;
        DECLARE rec tree_path;
        DECLARE pathes varchar[];
        DECLARE subpathes varchar[];
        DECLARE val varchar;
        declare i integer;
        BEGIN
            --get all ids from the source data
            for browse_rec in SELECT distinct(id) from tree_get_formated_data($1) LOOP    
                --for each id from the source datas, get all pathes
                for rec in SELECT * from tree_get_pathes(browse_rec.id, source) LOOP
                    RETURN NEXT rec;            
                end LOOP;
            end loop;
        RETURN;
        END;
        $BODY$
        LANGUAGE plpgsql VOLATILE;



        --
        --
        -- Example
        --
        --
        /*
        select path[2] from tree_get_all_pathes('SELECT aa.id as id, rel.parent_id, cast(aa.id as varchar) as val 
            FROM account_account aa LEFT JOIN account_account_rel rel ON (aa.id = rel.child_id)
            GROUP BY id, parent_id, val
            ORDER BY id') WHERE id = 759

        */

        """)
        cr.commit()
        
        
        
        
        #--------------------------------------------------------------------------------------
        # VIEWS
        #--------------------------------------------------------------------------------------
        
        
        # First draft Views for Excel reporting via ODBC
        # Show till two level of parent_id 
        # means that won't work for many2many
        # cr.execute("""create or replace view c2c_xls_reporting_account as 
        # 
        #         select 
        # 
        #             move.name as move,
        #             move_line.date,
        #             move_line.name as description,
        #             (cp_getsetofitemfromfullname(account.id))[1] as reporting_name,
        #             (cp_getsetofitemfromfullname(account.id))[2] as reporting_group,
        #             account.code || ' ' || account.name as account,
        #             move_line.debit,
        #             move_line.credit,
        #             move_line.debit-move_line.credit as balance,
        #             company_currency.name as company_currency,
        #             move_line.amount_currency,
        #             move_line_currency.name as move_currency,
        #             move_line.ref,
        #             journal.name as journal,
        #             period.name as period,
        #             company.name as company
        # 
        #         from     account_move_line as move_line left join res_currency as move_line_currency on move_line.currency_id = move_line_currency.id,
        #             account_move as move, account_journal as journal, account_period as period,
        #             res_company as company, account_account as account, res_currency as company_currency
        # 
        #         where
        #             move.id = move_line.move_id
        #             and move_line.journal_id = journal.id
        #             and move_line.period_id = period.id
        #             and move_line.account_id = account.id
        #             and account.company_id = company.id
        #             and company.currency_id = company_currency.id;""")
        
        
        # The brand new version with multi level and many2many support !
        # cr.execute("""create or replace view c2c_xls_reporting_account as 
        # select 
        # 
        #         move.name as move,
        #         move_line.date,
        #         move_line.name as description,
        #         (select code || ' ' || name from account_account where id=arnaud_req.p1::integer) as reporting_name,
        #         (select code || ' ' || name from account_account where id=arnaud_req.p2::integer) as reporting_group,
        #         account.code || ' ' || account.name as account,
        #         move_line.debit,
        #         move_line.credit,
        #         move_line.debit-move_line.credit as balance,
        #         company_currency.name as company_currency,
        #         move_line.amount_currency,
        #         move_line_currency.name as move_currency,
        #         move_line.ref,
        #         journal.name as journal,
        #         period.name as period,
        #         company.name as company
        # 
        #     from account_move_line as move_line left join res_currency as move_line_currency on move_line.currency_id = move_line_currency.id,
        #         account_move as move, account_journal as journal, account_period as period,
        #         res_company as company, account_account as account, res_currency as company_currency,
        #         (    select id as id,path[2] as p1,path[3] as p2 from tree_get_all_pathes('SELECT aa.id as id, 
        #             rel.parent_id, cast(aa.id as varchar) as val 
        #             FROM account_account aa LEFT JOIN account_account_rel rel ON (aa.id = rel.child_id)
        #             GROUP BY id, parent_id, val
        #             ORDER BY id')
        #         )as arnaud_req
        #         
        # 
        #     where
        #         move.id = move_line.move_id
        #         and move_line.journal_id = journal.id
        #         and move_line.period_id = period.id
        #         and move_line.account_id = account.id
        #         and account.company_id = company.id
        #         and company.currency_id = company_currency.id
        #         and account.id=arnaud_req.id
        # """)
        # cr.commit()
        
        
        # The brand new version with multi level and many2many support !

        # cr.execute("""create or replace view c2c_xls_reporting_aged_partner as 
        # SELECT line.name AS description, line.ref, p.name AS partner_name, COALESCE (c.name,'CHF') AS currency
        # , line.date, line.date_maturity, line.amount_currency, line.debit AS amount_chf
        #   FROM account_move_line line
        #   JOIN res_partner p ON p.id = line.partner_id
        #   left JOIN res_currency c ON c.id = line.currency_id
        #   , account_account
        #  WHERE line.account_id = account_account.id 
        #  AND account_account.type::text = 'receivable'::text 
        #  AND (line.reconcile_id IS NULL 
        #     OR (line.reconcile_id IN ( 
        #         SELECT line2.reconcile_id
        #         FROM account_move_line line2 
        #         WHERE line2.date >= now()
        #         ))) 
        #  AND line.partner_id IS NOT NULL 
        #  AND (line.date <= now() AND line.origin_link IS NULL 
        #  OR (line.id IN (
        #         SELECT line3.origin_link
        #          FROM account_move_line line3
        #          WHERE line3.origin_link IS NOT NULL AND line3.date <= now()))
        #  )
        #  ORDER BY line.date;
        # """)
        # cr.commit()
        
        
        
report_excel_c2c_base_function()
