# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008 Camptocamp SA
# @author Nicolas Bessi 
# @source JBA and AWST inpiration 
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from osv import osv, fields
import time
from mx import DateTime
from datetime import datetime, timedelta
import netsvc


class ProductCalculator(osv.osv):
    """Class that handle an ir cron call who will 
    recompute a table that contain unit price, price list id,
    product id, company id """
    _name = "xls.product.calculator"
    _description = "product calculator"
    ##dict that represent a cron object
    cron = {
            'active'          : False,
            'priority'        : 1,
            'interval_number' : 1,
            'interval_type'   : 'days',
            'nextcall'        : str(DateTime.today() + +DateTime.DateTimeDeltaFromDays(1.125)), #tomorrow 3:00 AM time
            'numbercall'      : -1,
            'doall'           : True,
            'model'           : 'xls.product.calculator',
            'function'        : 'run_product_compute',
            'args'            : '()',    
    }
        
    logger = netsvc.Logger()
    LOG_NAME = 'cron-rates'
    MOD_NAME = 'c2c_xls_reporting'
    _columns = {
        'product_id': fields.many2one('product.product','product_id',required = True),
        'price_list_id': fields.many2one('product.pricelist','price_list_id',required = True),
        'computed_unit_price': fields.float('unit_price',required = True),
        'company_id': fields.many2one('res.company','company_id',required = True),
        'update_date' : fields.datetime('update_date')
    }
    def get_cron_id(self, cr, uid, context):
        """return the updater cron's id. Create one if the cron does not exists """
        
        cron_id = 0
        cron_obj = self.pool.get('ir.cron')
        try: 
            #find the cron that send messages
            cron_id = cron_obj.search(
                                        cr, 
                                        uid,  
                                        [
                                            ('function', 'ilike', self.cron['function']), 
                                            ('model', 'ilike', self.cron['model'])
                                        ], 
                                        context={
                                                    'active_test': False
                                                } 
                                    )
            cron_id = int(cron_id[0])
        except Exception,e :
            self.logger.notifyChannel(
                                        self.LOG_NAME, 
                                        netsvc.LOG_INFO, 
                                        'warning cron not found one will be created'
                                     )
            print 'warning cron not found one will be created'
            pass # ignore if the cron is missing cause we are going to create it in db
        
        #the cron does not exists
        if not cron_id :
            #translate
            self.cron['name'] = _('Product Computation Update')
            cron_id = cron_obj.create(cr, uid, self.cron, context)
        
        return cron_id
        
    def save_cron(self, cr, uid, datas, context={}):
        """save the cron config data should be a dict"""
        #modify the cron
        cron_id = self.get_cron_id(cr, uid, context)        
        result = self.pool.get('ir.cron').write(cr, uid, [cron_id], datas)    

    def run_product_compute(self, cr, uid,mode='auto'):
        "update currency at the given frequence"
        if mode == 'auto' :
            if DateTime.now().hour != 3 :
                return
        multi_comp_pl = False
        now = DateTime.now()
        try :
            #if we are in a mutli company mode the company_id col should exsits
            cr.execute('select company_id from product_pricelist')
            cr.fetchall()
            enable = True
        except:
            cr.rollback()
            enable = False
        compagnies =  self.pool.get('res.company').search(cr, uid, [])
        calc_obj = self.pool.get('xls.product.calculator')
        #we compute all the product id to compute
        res=[]
        productids = self.pool.get('product.product').search(cr, uid, [])
        for comp in compagnies :
            for prod in self.pool.get('product.product').browse(cr, uid, productids) :
                if prod.company_id.id != comp and prod.company_id.id != False :
                    continue
                for price_list in self.pool.get('product.pricelist').search(cr, uid, []):
                    pl_br = self.pool.get('product.pricelist').browse(cr, uid, price_list)
                    if multi_comp_pl :
                        if pl_obj.company_id.id != prod.id :
                            continue
                    price = 0.0
                    try :
                        price = prod._get_price(pl_br.id, prod.id, 1.0)
                    except :
                        self.logger.notifyChannel(
                                            self.LOG_NAME, 
                                            netsvc.LOG_INFO, 
                                            'price was not updated for product %s'%(prod.name)
                                         )
                    to_update = calc_obj.search(
                                                    cr, 
                                                    uid, 
                                                    [
                                                        ('product_id','=',prod.id),
                                                        ('price_list_id', '=',pl_br.id ),
                                                        ('company_id','=', comp)
                                                    ]
                                                )
#                    self.logger.notifyChannel(
#                                            self.LOG_NAME, 
#                                            netsvc.LOG_INFO, 
#                                            'updating product %s'%(str(prod.id)),
#                                         )

                    if to_update :
                        to_update_id = calc_obj.browse(
                                                            cr, 
                                                            uid, 
                                                            to_update[0]
                                                        ).id
                        calc_obj.write(cr,uid,to_update_id, {'computed_unit_price':price})
                    else :
                        calc_obj.create(cr, uid, {
                                                    'product_id':prod.id,
                                                    'price_list_id':pl_br.id,
                                                    'company_id':comp,
                                                    'update_date':str(now),
                                                    'computed_unit_price':price,
                                                }
                                        )
                    cr.commit()
        self.logger.notifyChannel(
                                            self.LOG_NAME, 
                                            netsvc.LOG_INFO, 
                                            'product computation where updated at : %s'%(str(DateTime.now()))
                                         )
        return                
                        

                
ProductCalculator()
