# -*- encoding: utf-8 -*-
#
#  Created by Joel Grand-Guillaume
#
#  Copyright (c) 2009 CamptoCamp. All rights reserved.
##############################################################################
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
import time
from mx import DateTime
import netsvc
from osv import fields
from osv import osv
from datetime import datetime


# Class wich will create all view used by Excel reporting via ODBC
#   - Partners
#   - Contact
#   - Product
#   - Invoice line
#   - SO Lines
#   - Account move lines
#   - Account analytic line
class report_excel_c2c_base_view(osv.osv):
    _name = "report.excel.c2c.base.view"
    _description = "Base views for Excel reporting"
    _auto = False
    _log_access = False

    def init(self, cr):
        #delete view to avoid update problem
        try:
            cr.execute("DROP VIEW c2c_xls_reporting_partner_address CASCADE;")
            cr.commit()
        except:
            cr.rollback()
            
        # Partner and address: one line per address
        cr.execute("""create or replace view c2c_xls_reporting_partner_address as 
        SELECT 
            p.id as partner_id,
            p.ref as partner_code,
            p.name as partner_name,
            p.customer as iscustomer,
            p.supplier as issupplier,
            u.name as salesman,
            l.name as lang,
            a.id as address_id,
            a.type as address_type,
            a.street as address_street,
            a.street2 as address_street2,
            a.zip as address_zip,
            a.city as address_city,
            country.name as country,
            a.phone as address_phone,
            a.mobile as address_mobile,
            a.email as address_email
        FROM 
            res_partner_address a join res_country country 
            on (country.id=a.country_id) join res_partner p 
            on (p.id=a.partner_id) join res_lang l 
            on p.lang=l.code join res_users u 
            on u.id=p.user_id
        ORDER BY
            p.name asc
        """)
        cr.commit()
        
        #delete view to avoid update problem
        try:
            cr.execute("DROP VIEW c2c_xls_reporting_contact_address CASCADE;")
            cr.commit()
        except:
            cr.rollback()
        
        # Contact and address: one line per contact
        cr.execute("""create or replace view c2c_xls_reporting_contact_address as 
        SELECT 
            c.id as contact_id,
            c.name as contact_name,
            c.first_name as contact_firstname,
            l.name as contact_lang,
            j.id as job_id,
            f.name as job_function,
            j.phone as job_phone,
            j.fax as job_fax,
            j.email as job_email,
            j.state as job_state,
            j.date_start as job_date_start,
            j.date_stop as job_date_stop,
            j.address_id as address_id
        FROM 
            res_partner_contact c join res_partner_job j 
            on (c.id=j.contact_id) join res_partner_address a
            on (a.id=j.address_id) join res_lang l
            on (l.id=c.lang_id) join res_partner_function f
            on (f.id=j.function_id)

        ORDER BY
            c.name asc
        """)
        cr.commit()

        #delete view to avoid update problem
        try:
            cr.execute("DROP VIEW c2c_xls_reporting_invoices CASCADE;")
            cr.commit()
        except:
            cr.rollback()

        # Invoice and Invoices
        cr.execute("""create or replace view c2c_xls_reporting_invoices as 
        SELECT 
            i.id as inv_id,
            i.name as inv_name,
            i.origin as inv_origin,
            i.date_due as inv_date_due,
            i.date_invoice as inv_date_invoice,
            i.reference as inv_reference,
            i.payment_term as inv_payment_term_id,
            account_payment_term.name as inv_payment_term_name,
            i.number as inv_number,
            i.journal_id as inv_journal_id,
            account_journal.name as inv_journal_name,
            i.currency_id as inv_currency_id,
            res_currency.code as inv_currency_name,
            i.company_id as inv_company_id,
            res_company.name as inv_company_name,
            i.partner_id as inv_partner_id,
            res_partner.ref as inv_partner_ref,
            res_partner.name as inv_partner_name,
            i.address_invoice_id as inv_address_invoice_id,
            res_partner_address.name as inv_address_invoice_name,
            i.account_id as inv_account_id,
            account_account.code as inv_account_code,
            account_account.name as inv_account_name,
            i.fiscal_position as inv_fiscal_position,
            account_fiscal_position.name as inv_fiscal_position_name,
            i.amount_untaxed as inv_amount_untaxed,
            i.amount_tax as inv_amount_tax,
            i.amount_total as inv_amount_total,
            i.state as inv_state,
            i.type as inv_type,
            i.reconciled as inv_reconciled,
            i.residual as inv_residual,
            i.period_id as inv_period_id,
            account_period.name as inv_period_name,
            account_period.code as inv_period_code,
            account_fiscalyear.name as inv_fiscalyear_name,
            i.move_id as inv_move_id,
            i.price_type as inv_price_type
        FROM 
            account_invoice i
            left join account_payment_term on (account_payment_term.id=i.payment_term)
            join account_journal on (account_journal.id=i.journal_id)
            join res_currency on (res_currency.id=i.currency_id)
            join res_partner on (res_partner.id=i.partner_id)
            join res_partner_address on (res_partner_address.id=i.address_invoice_id)
            join account_account on (account_account.id=i.account_id)
            left join account_fiscal_position on (account_fiscal_position.id=i.fiscal_position)
            join res_company on (res_company.id=i.company_id)
            left join account_period on (account_period.id=i.period_id)
            left join account_fiscalyear on (account_fiscalyear.id=account_period.fiscalyear_id)
            order by i.id
        """)
        cr.commit()        #delete view to avoid update problem
        try:
            cr.execute("DROP VIEW c2c_xls_reporting_invoice_lines CASCADE;")
            cr.commit()
        except:
            cr.rollback()

        # Invoice and Invoice Lines
        cr.execute("""create or replace view c2c_xls_reporting_invoice_lines as 
        SELECT 
            l.id as inv_line_id,
            l.origin as inv_line_origin,
            l.uos_id as inv_line_uos_id,
            l.name as inv_line_name,
            l.price_unit as inv_line_price_unit,
            l.price_subtotal as inv_line_price_subtotal,
            l.account_id as inv_line_account_id,
            account_account.code as inv_line_account_code,
            account_account.name as inv_line_account_name,
            l.discount as inv_line_discount,
            l.account_analytic_id as inv_line_account_analytic_id,
            account_analytic_account.code as inv_line_account_analytic_code,
            account_analytic_account.name as inv_line_account_analytic_name,
            l.quantity as inv_line_quantity,
            l.product_id as inv_line_product_id,
            product_product.default_code as inv_line_product_code,
            product_template.name as inv_line_product_name,
            l.price_subtotal_incl as inv_line_price_subtotal_incl,
            c2c_xls_reporting_invoices.*
        FROM
            account_invoice_line l 
            join c2c_xls_reporting_invoices on (l.invoice_id=c2c_xls_reporting_invoices.inv_id)
            join account_account on (account_account.id=l.account_id)
            left join account_analytic_account on (account_analytic_account.id=l.account_analytic_id)
            left join product_product on (product_product.id=l.product_id)
            left join product_template on (product_template.id=product_product.product_tmpl_id)
        """)
        cr.commit()
        
        
        #delete view to avoid update problem
        try:
            cr.execute("DROP VIEW c2c_xls_reporting_sale_lines CASCADE;")
            cr.commit()
        except:
            cr.rollback()
        
        
         # Sale and Sales Lines
        cr.execute("""create or replace view c2c_xls_reporting_sale_lines as 
        SELECT 
             sale_order_line.id AS so_line_id, 
             sale_order_line.name AS so_line_name, 
             sale_order_line.price_unit AS so_line_price_unit, 
             sale_order_line.price_subtotal AS so_line_price_subtotal, 
             sale_order_line.price_subtotal_incl AS so_line_price_subtotal_incl, 
             sale_order_line.product_uom_qty AS so_line_product_uom_qty,              
             sale_order_line.product_uom AS so_line_product_uom_id, 
             uom.name AS so_line_product_uom_name, 
             sale_order_line.product_uos_qty AS so_line_product_uos_qty,
             sale_order_line.product_uos AS so_line_product_uos_id, 
             uos.name AS so_line_product_uos_name, 
             sale_order_line.product_id AS so_line_product_id, 
             product_product.default_code AS so_line_product_code, 
             product_template.name AS so_line_product_name, 
             sale_order_line.state AS so_line_state, 
             sale_order.id AS so_id, 
             sale_order.name AS so_name, 
             product_pricelist.id AS so_pricelist_id, 
             product_pricelist.name AS so_pricelist_name, 
             res_currency.name AS so_currency, 
             sale_order.state AS so_state, 
             sale_order.origin AS so_origin, 
             sale_order.date_order AS so_date_order, 
             res_partner.name AS so_partner_name, 
             sale_order.user_id AS so_user_id, 
             res_users.name AS so_sales_man, 
             sale_shop.name AS so_shop_name,
             sale_order.project_id AS so_aa_id, 
             account_analytic_account.code AS so_aa_code, 
             account_analytic_account.name AS so_aa_name
           FROM 
             public.sale_order join public.sale_order_line on (sale_order.id=sale_order_line.order_id)
             left join public.product_product on (product_product.id=sale_order_line.product_id)
             join public.product_template on (product_template.id=product_product.product_tmpl_id)
             join public.product_pricelist on (product_pricelist.id=sale_order.pricelist_id)
             join public.res_currency on (res_currency.id=product_pricelist.currency_id)
             join public.res_partner on (res_partner.id = sale_order.partner_id)
             join public.sale_shop on (sale_shop.id=sale_order.shop_id)
             left join public.account_analytic_account on (account_analytic_account.id=sale_order.project_id)
             join public.res_users on (res_users.id=sale_order.user_id)
             left join product_uom uom on (uom.id=sale_order_line.product_uom)
             left join product_uom uos on (uos.id=sale_order_line.product_uos)
           ORDER BY sale_order.name asc ;
        """)
        cr.commit()
        
        #delete view to avoid update problem
        try:
            cr.execute("DROP VIEW c2c_xls_reporting_acc_analytic_lines CASCADE;")
            cr.commit()
        except:
            cr.rollback()
        
      
        # Account Analytic Line
        cr.execute("""create or replace view c2c_xls_reporting_acc_analytic_lines as 
        SELECT 
          account_analytic_line.id AS aa_line_id, 
          account_analytic_line.code AS aa_line_code, 
          account_analytic_line.user_id AS aa_line_user_id,
          ul.name AS aa_line_user_name,
          ul.company_id as aa_line_user_company_id,
          ul_comp.name as aa_line_user_company_name,
          account_analytic_line."name" AS aa_line_name, 
          account_analytic_line.general_account_id AS aa_line_general_account_id, 
          account_account.code AS aa_line_general_account_code, 
          account_account.name AS aa_line_general_account_name, 
          account_analytic_line.journal_id AS aa_line_journal_id, 
          account_analytic_journal.name AS aa_line_journal_name, 
          account_analytic_journal.type AS aa_line_journal_type, 
          account_analytic_line.amount AS aa_line_amount, 
          account_analytic_line.product_id AS aa_line_product_id, 
          product_product.default_code AS aa_line_product_code, 
          product_template.name AS aa_line_product_name, 
          account_analytic_line.unit_amount AS aa_line_unit_amount, 
          account_analytic_line.date AS aa_line_date, 
          account_analytic_line.ref AS aa_line_ref, 
          account_analytic_line.move_id AS aa_line_move_id, 
          account_analytic_line.account_id AS aa_id, 
          account_analytic_account.code AS aa_code, 
          account_analytic_account."name" AS aa_name, 
          account_analytic_account."type" AS aa_type, 
          (cp_getsetofitemfromfullname(account_analytic_line.account_id))[1] as parent_level1,
          (cp_getsetofitemfromfullname(account_analytic_line.account_id))[2] as parent_level2,
          (cp_getsetofitemfromfullname(account_analytic_line.account_id))[3] as parent_level3,
          (cp_getsetofitemfromfullname(account_analytic_line.account_id))[4] as parent_level4,
          (cp_getsetofitemfromfullname(account_analytic_line.account_id))[5] as parent_level5,
          account_analytic_account.quantity_max AS aa_quantity_max, 
          account_analytic_account.partner_id AS aa_partner_id, 
          res_partner."name" AS aa_partner_name, 
          res_partner.ref AS aa_partner_ref, 
          account_analytic_account.user_id AS aa_user_id,
          user_account.name AS aa_user_name,
          account_analytic_account.company_id AS aa_company_id,
          res_company.name AS aa_company_name

        FROM 
          public.account_analytic_line join public.account_analytic_account
          on (account_analytic_line.account_id = account_analytic_account.id)
          left join res_users as ul on (ul.id=account_analytic_line.user_id)
          left join res_company as ul_comp on (ul_comp.id=ul.company_id)
          join account_account on (account_account.id=account_analytic_line.general_account_id)
          join account_analytic_journal on (account_analytic_journal.id=account_analytic_line.journal_id)
          join product_product on (product_product.id=account_analytic_line.product_id)
          join public.product_template on (product_template.id=product_product.product_tmpl_id)
          left join public.res_partner on (res_partner.id=account_analytic_account.partner_id)
          join res_users user_account on (user_account.id=account_analytic_account.user_id)
          join res_company on (res_company.id=account_analytic_account.company_id)
        ORDER BY account_analytic_account.name asc
        """)
        cr.commit()
        
        #delete view to avoid update problem
        try:
            cr.execute("DROP VIEW c2c_xls_reporting_acc_move_lines CASCADE;")
            cr.commit()
        except:
            cr.rollback()
        
        # Account Move Line
        cr.execute("""create or replace view c2c_xls_reporting_acc_move_lines as 
        SELECT 
          account_move_line.id AS aml_id, 
          account_move_line.reconcile_id AS aml_reconcile_id, 
          account_move_line.statement_id AS aml_statement_id, 
          account_move_line.tax_amount AS aml_tax_amount,
          account_move_line.debit AS aml_debit, 
          account_move_line.credit AS aml_credit, 
          account_move_line.debit - account_move_line.credit AS aml_balance, 
          account_move_line.amount_currency AS aml_amount_currency,
          account_move_line.currency_id AS aml_currency_id,  
          res_currency.name AS aml_currency_name, 
          cur.name as aml_company_currency_name,
          account_move_line.account_id AS aml_account_id, 
          account_account.code AS aml_account_code, 
          account_account.name AS aml_account_name,
          (cp_getsetofaccountfromfullname(account_move_line.account_id))[1] as parent_level1,
          (cp_getsetofaccountfromfullname(account_move_line.account_id))[2] as parent_level2,
          (cp_getsetofaccountfromfullname(account_move_line.account_id))[3] as parent_level3,
          (cp_getsetofaccountfromfullname(account_move_line.account_id))[4] as parent_level4,
          (cp_getsetofaccountfromfullname(account_move_line.account_id))[5] as parent_level5,
          (cp_getsetofaccountfromfullname(account_move_line.account_id))[6] as parent_level6,
          (cp_getsetofaccountfromfullname(account_move_line.account_id))[7] as parent_level7,
          (cp_getsetofaccountfromfullname(account_move_line.account_id))[8] as parent_level8,
          account_move_line.date_maturity AS aml_date_maturity, 
          account_move_line.period_id AS aml_period_id, 
          account_period.code AS aml_period_code,
          account_period.name AS aml_period_name, 
          account_fiscalyear.name as aml_fiscal_year_name,
          account_move_line.date AS aml_date, 
          account_move_line.partner_id AS aml_partner_id, 
          res_partner.ref AS aml_partner_ref, 
          res_partner.name AS aml_partner_name, 
          account_move_line.reconcile_partial_id AS aml_reconcile_partial_id, 
          account_move_line.analytic_account_id AS aml_analytic_acc_id, 
          account_analytic_account.code AS aml_analytic_acc_code, 
          account_analytic_account.name AS aml_analytic_acc_name, 
          account_move_line."name" AS aml_name, 
          account_move_line.account_tax_id AS aml_account_tax_id, 
          account_tax.type_tax_use AS aml_account_tax_type_tax_use, 
          account_tax.name AS aml_account_tax_name, 
          account_move_line.journal_id AS aml_journal_id, 
          account_journal.name as aml_journal_name,
          account_move_line.tax_code_id AS aml_tax_code_id, 
          account_tax_code.code AS aml_tax_code_code, 
          account_tax_code.name AS aml_tax_code_name, 
          account_move_line.state AS aml_state, 
          account_move_line.product_id AS aml_product_id, 
          product_product.default_code AS aml_product_code, 
          product_template.name AS aml_product_name, 
          account_move_line.move_id AS aml_move_id, 
          account_move.name AS aml_move_name, 
          account_move_line.ref AS aml_ref
        FROM 
          public.account_move_line
          join account_account on (account_account.id=account_move_line.account_id)
          join account_period on (account_period.id=account_move_line.period_id)
          join account_fiscalyear on (account_fiscalyear.id=account_period.fiscalyear_id)
          join account_journal on (account_journal.id = account_move_line.journal_id)
          left join res_currency on (res_currency.id=account_move_line.currency_id)
          left join res_partner on (res_partner.id=account_move_line.partner_id)
          left join account_analytic_account on (account_analytic_account.id=account_move_line.analytic_account_id)
          left join account_tax on (account_tax.id=account_move_line.account_tax_id)
          left join account_tax_code on (account_tax_code.id=account_move_line.tax_code_id)
          left join product_product on (product_product.id=account_move_line.product_id)
          left join public.product_template on (product_template.id=product_product.product_tmpl_id)
          left join account_move on (account_move.id=account_move_line.move_id),
          res_company comp join res_currency cur on (comp.currency_id=cur.id)
        ORDER BY account_move_line.date
        
        """)
        cr.commit()
      
        #delete view to avoid update problem
        try:
            cr.execute("DROP VIEW c2c_xls_reporting_product_product CASCADE;")
            cr.commit()
        except:
            cr.rollback()
      
        # product_product
        cr.execute("""create or replace view c2c_xls_reporting_product_product as 
        SELECT 
          product_product.id AS product_id, 
          product_product.default_code AS product_code, 
          product_product.variants AS product_variant, 
          product_template."name" AS product_name, 
          product_template.list_price AS product_list_price, 
          product_template.standard_price AS product_standard_price, 
          product_template.uom_id as product_uom_id,
          product_uom.name as product_uom_name,
          product_template.product_manager AS product_manager_id, 
          res_users.name AS product_manager_name, 
          product_template.company_id AS product_company_id, 
          res_company.name AS product_company_name, 
          product_template.state AS product_state, 
          product_template."type" AS product_type, 
          product_template.categ_id AS product_categ_id,
          product_category.name as product_categ_name,
          (cp_getsetproductcategfullname(product_template.categ_id))[1] as parent_level1,
          (cp_getsetproductcategfullname(product_template.categ_id))[2] as parent_level2,
          (cp_getsetproductcategfullname(product_template.categ_id))[3] as parent_level3,
          (cp_getsetproductcategfullname(product_template.categ_id))[4] as parent_level4,
          (cp_getsetproductcategfullname(product_template.categ_id))[5] as parent_level5
        FROM 
          product_product join public.product_template on (product_template.id=product_product.product_tmpl_id)
          join product_category on (product_category.id=product_template.categ_id)
          left join res_users on (res_users.id=product_template.product_manager)
          left join res_company on (res_company.id=product_template.company_id)
          join product_uom on (product_uom.id=product_template.uom_id)
        ORDER BY product_template.name asc
        """)
        cr.commit()

        #delete view to avoid update problem
        try:
            cr.execute("DROP VIEW c2c_xls_reporting_analytic_account CASCADE;")
            cr.commit()
        except:
            cr.rollback()
      
        # account_analytic_account
        cr.execute("""create or replace view c2c_xls_reporting_analytic_account as 
        SELECT 
          account_analytic_account.id AS aa_id, 
          account_analytic_account.code AS aa_code, 
          account_analytic_account.name AS aa_name, 
          account_analytic_account.type AS aa_type, 
          account_analytic_account.state AS aa_state, 
          account_analytic_account.quantity_max AS aa_quantity_max, 
          account_analytic_account.date AS aa_date, 
          account_analytic_account.date_start AS aa_date_start, 
          account_analytic_account.contact_id as aa_contact_id,
          res_partner_address.name as aa_contact_name,
          account_analytic_account.partner_id as aa_partner_id,
          res_partner.ref as aa_partner_ref,
          res_partner.name as aa_partner_name,
          account_analytic_account.user_id as aa_user_id,
          res_users.name as aa_user_name,
          account_analytic_account.company_id as aa_company_id,
          res_company.name as aa_company_name,
          (cp_getsetofitemfromfullname(account_analytic_account.id))[1] as parent_level1,
          (cp_getsetofitemfromfullname(account_analytic_account.id))[2] as parent_level2,
          (cp_getsetofitemfromfullname(account_analytic_account.id))[3] as parent_level3,
          (cp_getsetofitemfromfullname(account_analytic_account.id))[4] as parent_level4,
          (cp_getsetofitemfromfullname(account_analytic_account.id))[5] as parent_level5
        FROM 
          account_analytic_account 
          left join res_partner_address on (account_analytic_account.contact_id=res_partner_address.id)
          left join res_partner on (res_partner.id=account_analytic_account.partner_id)
          left join res_users on (res_users.id=account_analytic_account.user_id)
          left join res_company on (res_company.id=account_analytic_account.company_id)
        ORDER BY account_analytic_account.name asc
        """)
        cr.commit()

        #delete view to avoid update problem
        try:
            cr.execute("DROP VIEW c2c_xls_reporting_account_account CASCADE;")
            cr.commit()
        except:
            cr.rollback()
      
        # account_account
        cr.execute("""create or replace view c2c_xls_reporting_account_account as 
        SELECT 
          account_account.id AS account_id, 
          account_account.code AS account_code, 
          account_account.name AS account_name, 
          account_account.currency_id as account_currency_id,
          res_currency.name as account_currency_name,
          account_account.currency_mode as account_currency_mode,
          account_account.user_type as account_user_type,
          account_account.type as account_type,
          account_account.company_id as account_company_id,
          res_company.name as account_company_name,
          account_account.reconcile as account_reconcile,
          account_account.check_history as account_check_history,
          (cp_getsetofaccountfromfullname(account_account.id))[1] as parent_level1,
          (cp_getsetofaccountfromfullname(account_account.id))[2] as parent_level2,
          (cp_getsetofaccountfromfullname(account_account.id))[3] as parent_level3,
          (cp_getsetofaccountfromfullname(account_account.id))[4] as parent_level4,
          (cp_getsetofaccountfromfullname(account_account.id))[5] as parent_level5,
          (cp_getsetofaccountfromfullname(account_account.id))[6] as parent_level6,
          (cp_getsetofaccountfromfullname(account_account.id))[7] as parent_level7,
          (cp_getsetofaccountfromfullname(account_account.id))[8] as parent_level8
        FROM 
          account_account 
          left join res_currency on (res_currency.id=account_account.currency_id)
          left join res_company on (res_company.id=account_account.company_id)
          
        ORDER BY account_account.name asc
        """)
        cr.commit()
        

      
        
report_excel_c2c_base_view()